using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace LLNet.Controls
{
	/// <summary>
	/// Summary description for LLAdvancedTreeList.
	/// </summary>
	public class LLAdvancedTreeList : DevExpress.XtraEditors.ButtonEdit 
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private Model.IBLL bllInstance;

		private Utils.QueryParams externPList;
		
		private string connectionName;

		public event AfterGetRowEventHandler AfterGetRow;

		public LLAdvancedTreeList()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call
			this.ButtonClick +=new DevExpress.XtraEditors.Controls.ButtonPressedEventHandler(LLAdvancedTreeList_ButtonClick);
		
		

		}

		private dlgAdvancedTreeList dlg;

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Properties.Buttons.AddRange(new DevExpress.XtraEditors.Controls.EditorButton[] {
																									new DevExpress.XtraEditors.Controls.EditorButton()});
			this.Properties.Name = "Properties";
			this.Size = new System.Drawing.Size(75, 20);
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.LLAdvancedTreeList_KeyDown);
			this.Validating += new System.ComponentModel.CancelEventHandler(this.LLAdvancedTreeList_Validating);

		}
		#endregion


		#region public properties

		public Model.IBLL BllInstance
		{
			get {return bllInstance;}
			set {bllInstance = value;}
		}

		public Utils.QueryParams QueryParams
		{
			get {return externPList;}
			set {externPList = value;}
		}

		public string ConnectionName
		{
			get {return connectionName;}
			set {connectionName=value;}
		}
		#endregion

		private void OpenListOfValues()
		{
			object editValue;
			object tag;

			editValue = this.EditValue;
			tag = this.Tag;
			
			dlg = new dlgAdvancedTreeList();
			dlg.dataView = this.DataBindings["EditValue"].DataSource as DataView;
			dlg.dataView.Table.Clear();
			dlg.llToolbar.SetDataBinding(dlg.dataView);

			foreach(System.Data.DataColumn col in dlg.dataView.Table.Columns)
			{
				if (col.Caption == "Key")
					dlg.Grid.KeyFieldName = col.ColumnName;

				if (col.Caption == "Value")
					dlg.Grid.ParentFieldName = col.ColumnName;

				if (col.Caption == "Display")
				{
					dlg.Grid.PreviewFieldName = col.ColumnName;
					col.Caption = col.ColumnName;
				}
			}

			dlg.Grid.PopulateColumns();

			dlg.Grid.DataSource = dlg.dataView;

			dlg.llToolbar.Open(bllInstance,connectionName,externPList);

						

			if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
			{
				DataRowView drv = null;
				if (dlg.BindingContext[dlg.dataView].Count > 0)
				{
					drv = dlg.BindingContext[dlg.dataView].Current as DataRowView;
					this.Tag = drv.Row;
					this.EditValue = drv.Row[DataBindings["EditValue"].BindingMemberInfo.BindingField].ToString();
				}

				if (AfterGetRow != null)
				{
					AdvancedListEventArgs aea = new AdvancedListEventArgs();
					if (dlg.BindingContext[dlg.dataView].Count > 0)
					{
						aea.Row = drv.Row;
					}
					AfterGetRow(this,aea);
					//this.Tag = drv.Row[DataBindings["Tag"].BindingMemberInfo.BindingField].ToString();
					
				}
			}
			else
			{
				this.Tag = tag;
				this.EditValue = editValue;
				
			}

		}

		private void LLAdvancedTreeList_ButtonClick(object sender, DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e)
		{
			OpenListOfValues();

		}

		private void getRow()
		{
			Utils.QueryParams oldExternPList = externPList;

			string pValue = this.DataBindings["EditValue"].BindingMemberInfo.BindingField.ToString();
			Utils.QueryParams qPrms = new LLNet.Utils.QueryParams();

			string vParam = "'" + this.EditValue.ToString() + "'";
			qPrms.AddRecursiveConditionParam(Utils.QueryParams.PredefinedTags.AllToolbarsTag,"AND",pValue,"=",vParam);
			externPList= qPrms;
					
			dlg = new dlgAdvancedTreeList();
			dlg.dataView = this.DataBindings["EditValue"].DataSource as DataView;
			dlg.llToolbar.SetDataBinding(dlg.dataView);
			dlg.Grid.DataSource = dlg.dataView;
			dlg.llToolbar.Open(bllInstance,connectionName,externPList);

			externPList = oldExternPList;

			DataRowView drv = null;

			if (dlg.BindingContext[dlg.dataView].Count > 0)
			{
				drv = dlg.BindingContext[dlg.dataView].Current as DataRowView;
				this.Tag = drv.Row;
			}
			if (AfterGetRow != null)
			{
				AdvancedListEventArgs aea = new AdvancedListEventArgs();

				if (dlg.BindingContext[dlg.dataView].Count > 0)
				{
					aea.Row = drv.Row;
				}
				AfterGetRow(this,aea);
			}

		}
		
		private void LLAdvancedTreeList_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			switch ( e.KeyCode)
			{
				case Keys.Enter:
					//getRow();
					break;
				case Keys.F4:
					OpenListOfValues();
					break;
			}
		}

		private void LLAdvancedTreeList_Validating(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (Utils.Helpers.ObjToStr(this.EditValue)!="")
			{
				getRow();
			}
			else
			{
				this.EditValue = null;
			}
		}
		
	}
}
