using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace LLNet.Tienda.Dialogs.Refunds
{

    public partial class dlgGiftCard : GUI.FormBases.frmBase
    {
        private decimal? sku;
        private decimal? valePadre;
        private decimal? saldo;
        public decimal? ValeGenerado;
        private string tipoVale;

        #region Constructor

        public dlgGiftCard()
        {
            InitializeComponent();

            GUI.CApp.TraducirForm(this);
        }

        #endregion

        public override void Init(object[] pArgs)
        {
            base.Init(pArgs);
            if (pArgs != null)
            {
                saldo = Convert.ToDecimal(pArgs[0]);
                valePadre = Convert.ToDecimal(pArgs[1]);
                tipoVale = pArgs[2].ToString();
                rowMonto.Properties.Value = saldo;
                rowMonto.Properties.ReadOnly = true;
                merProducto.Enabled = false;
            }
            else
                tipoVale = "A";
        }

        #region Eventos

        private void btnAceptar_Click(object sender, EventArgs e)
        {
            vGridControl1.PostEditor();
            vGridControl1.UpdateFocusedRecord();
            vGridControl1.FocusFirst();

            if (rowVale.Properties.Value == null || rowVale.Properties.Value == DBNull.Value)
            {
                GUI.CApp.ShowMessage("Ingrese nmero de vale", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowVale;
                return;
            }

            Utils.QueryParams qp = new LLNet.Utils.QueryParams();
            qp.AddReplaceParam("VALE", rowVale.Properties.Value.ToString());

            if (GUI.CApp.bll.GetSingleValue("ExisteVale", qp, "PDV").ToString() == "1")
            {
                GUI.CApp.ShowMessage("El nmero de certificado de regalo ya existe", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowVale;
                vGridControl1.Focus();
                return;
            }

            if (Utils.Helpers.ObjToStr(GUI.CApp.bll.GetSingleValue("ValidaRangoVale", qp, "PDV")) == "0")
            {
                GUI.CApp.ShowMessage("El nmero de vale no se encuentra dentro del rango permitido para la tienda", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowVale;
                vGridControl1.Focus();
                return;
            }

            if (rowVencimiento.Properties.Value != null || rowVencimiento.Properties.Value != DBNull.Value)
            {
                if (Convert.ToDateTime(rowVencimiento.Properties.Value) < DateTime.Today)
                {
                    GUI.CApp.ShowMessage("La fecha de vencimiento no es vlida", "Certificados de Regalo", true);
                    vGridControl1.FocusedRow = rowVencimiento;
                    vGridControl1.Focus();
                    return;
                }
            }
            else
            {
                GUI.CApp.ShowMessage("La fecha de vencimiento no es vlida", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowVencimiento;
                vGridControl1.Focus();
                return;
            }

            if (rowPara.Properties.Value == null || rowPara.Properties.Value == DBNull.Value)
            {
                GUI.CApp.ShowMessage("Ingrese el nombre de la persona que recibir el certificado de regalo", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowPara;
                return;
            }

            if (rowMonto.Properties.Value == null || rowMonto.Properties.Value == DBNull.Value)
            {
                GUI.CApp.ShowMessage("Ingrese monto del certificado de regalo", "Certificados de Regalo", true);
                vGridControl1.FocusedRow = rowMonto;
                return;
            }

            if (rowVencimiento.Properties.Value == null || rowVencimiento.Properties.Value == DBNull.Value)
            {
                vGridControl1.FocusedRow = rowVencimiento;
                return;
            }

            #region Insertando certificado de regalo

            decimal vSku = Convert.ToDecimal(GUI.CApp.GetParam("SPIC"));
            qp.Clear();

            qp.AddReplaceParam("SKU", vSku.ToString());

            if (GUI.CApp.bll.GetSingleValue("FindSku",qp,"PDV").ToString() != "0")
            {
                if (Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS.Count == 0)
                {
                    if (Program.FrmMainPos.pos.ctlAddSkus1.CreateOrder(null) == -1)
                        return;

                    Program.FrmMainPos.RefreshData();
                }

                if (Utils.Helpers.ObjToStr(GUI.CApp.GetParam("PVCF")).Trim() == string.Empty)
                {
                    if (Utils.Helpers.ObjToStr(Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS[0]["SOCIO"]) == Utils.Helpers.ObjToStr(GUI.CApp.GetParam("NSCF")))
                    {
                        GUI.CApp.ShowMessage("No puede crear un certificado de regalo a consumidor final", "Certificados de Regalo", true);
                        return;
                    }
                }

                Model.PDV.DsPDV.DOC_VALESRow drw = Program.FrmMainPos.DsVale.DOC_VALES.NewDOC_VALESRow();
                drw.VALE = Convert.ToDecimal(rowVale.Properties.Value);
                drw.DE_PARTE_DE = Utils.Helpers.ObjToStr(rowDe.Properties.Value);
                drw.A_NOMBRE_DE = Utils.Helpers.ObjToStr(rowPara.Properties.Value);
                drw.VENCE_EL = Convert.ToDateTime(rowVencimiento.Properties.Value);
                drw.TIPO = (tipoVale ?? "A");
                drw.ESTADO = "P";                                
                drw.MONTO = Convert.ToDecimal(rowMonto.Properties.Value);

                drw.JORNADA = (decimal)Program.Jornada;

                drw.CREADO_POR = GUI.CApp.usuario;
                drw.CREADO_EL = DateTime.Now;
                
                if (valePadre != null)
                {
                    drw.VALE_PADRE = (decimal)valePadre;
                    ValeGenerado = Convert.ToDecimal(rowVale.Properties.Value);
                }

                drw.SOCIO = Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS[0].SOCIO;
                if (sku != null)
                    drw.SKU = (decimal)sku;
                Program.FrmMainPos.DsVale.DOC_VALES.AddDOC_VALESRow(drw);

                GUI.CApp.bll.updateDataSet(Program.FrmMainPos.DsVale, Program.FrmMainPos.DsVale.DOC_VALES.TableName);

                if (valePadre == null)
                {
                    qp.Clear();
                    qp.AddReplaceParam("SKU", vSku.ToString());
                    qp.AddReplaceParam("SOCIO", drw["SOCIO"].ToString());

                    decimal vPctImp = Convert.ToDecimal(GUI.CApp.bll.GetSingleValue("GetTaxes", qp, "PDV"));
                    decimal vPrecio = decimal.Parse(rowMonto.Properties.Value.ToString());
                    decimal vPrecioUnitario = vPrecio / (1 + (vPctImp / 100));
                    decimal vImpuesto = vPrecio - vPrecioUnitario;

                    Program.FrmMainPos.pos.ctlAddSkus1.InsertOrderLine(null, 1, GUI.CApp.usuario, vPrecioUnitario, vImpuesto,
                         null, null, vSku, decimal.Parse(rowVale.Properties.Value.ToString()), null, null);
                }
            }

            Program.FrmMainPos.RefreshData();

            #endregion
    
            DialogResult = DialogResult.OK;
        }

        private void btnCorrelativo_Click(object sender, EventArgs e)
        {
            rowVale.Properties.Value = GUI.CApp.GetSecuence("DOC_S_VALE");
            vGridControl1.FocusedRow = rowDe;
            vGridControl1.ShowEditor();
        }

        #endregion

        private void dlgGiftCard_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Escape:
                    btnCancelar.PerformClick();
                    break;
                case Keys.F2:
                    btnCorrelativo_Click(null, null);
                    break;
                case Keys.F12:
                    if (btnAceptar.Enabled)
                        btnAceptar_Click(null, null);
                    break;
            }
        }

        private void vGridControl1_CellValueChanged(object sender, DevExpress.XtraVerticalGrid.Events.CellValueChangedEventArgs e)
        {
            if (e.Row.Name == "merProducto")
            {
                selectProduct(e);

            }
        }

        private void selectProduct(DevExpress.XtraVerticalGrid.Events.CellValueChangedEventArgs e)
        {
            object objValue; 

            if (e != null)
            {
                objValue = ((DevExpress.XtraVerticalGrid.Rows.MultiEditorRow)e.Row).PropertiesCollection[0].Value;
            }
            else
            {
                objValue = merProducto.PropertiesCollection[0].Value;
            }

            if (Utils.Helpers.ObjToStr(objValue) != string.Empty)
            {
                if (Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS.Count == 0)
                {
                    if (Program.FrmMainPos.pos.ctlAddSkus1.CreateOrder(null) == -1)
                        return;

                    Program.FrmMainPos.RefreshData();
                }

                if (FindSku(objValue.ToString(),
                    Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS[0].SOCIO, GUI.CApp.usuario))
                {
                    merProducto.PropertiesCollection[1].Value = dsPDV1.WMS_SKUS[0].DESCRIPCION;
                    rowMonto.Properties.Value = dsPDV1.WMS_SKUS[0].PRECIO_BASE;
                    sku = dsPDV1.WMS_SKUS[0].SKU;
                    rowMonto.Properties.ReadOnly = true;
                }
                else
                {
                    if (e != null)
                        ((DevExpress.XtraVerticalGrid.Rows.MultiEditorRow)e.Row).PropertiesCollection[0].Value = null;
                    else
                        merProducto.PropertiesCollection[0].Value = null;

                    merProducto.PropertiesCollection[1].Value = null;
                    rowMonto.Properties.ReadOnly = false;
                }
            }
            else
            {
                sku = null;
                ((DevExpress.XtraVerticalGrid.Rows.MultiEditorRow)e.Row).PropertiesCollection[1].Value = null;
                rowMonto.Properties.ReadOnly = false;
            }
        }

        public bool FindSku(string pSkuAlterno, decimal pSocio, string pUsuario)
        {

            Utils.QueryParams qp = new LLNet.Utils.QueryParams();
            qp.AddReplaceParam("SOCIO", pSocio.ToString());
            qp.AddReplaceParam("USUARIO", pUsuario);

            if (GUI.CApp.GetParam("HBMM").ToString() == "X")
            {
                qp.AddReplaceParam("SKU_ALTERNO", pSkuAlterno.ToUpper());
            }
            else
            {
                qp.AddReplaceParam("SKU_ALTERNO", pSkuAlterno);
            }

            qp.AddReplaceParam("DOCUMENTO", Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS[0].DOCUMENTO.ToString());            

            dsPDV1.WMS_SKUS.Clear();
            Utils.Helpers.Merge(GUI.CApp.bll.getTypedDataSet(dsPDV1.GetType(), dsPDV1.WMS_SKUS.TableName, qp, "GetSkus"), dsPDV1);

            return (dsPDV1.WMS_SKUS.Rows.Count > 0);
        }

        private void repSkuAlterno_ButtonClick(object sender, DevExpress.XtraEditors.Controls.ButtonPressedEventArgs e)
        {
            MainForms.Finders.frmFindSku ffs = new LLNet.Tienda.MainForms.Finders.frmFindSku();
            ffs.Init(null);
            ffs.ShowDialog();

            if (ffs != null)
            {
                if (ffs.DialogResult == DialogResult.OK)
                {
                    merProducto.PropertiesCollection[0].Value = ffs.SkuAlterno;
                    selectProduct(null);
                }
            }            
        }

        private void dlgGiftCard_Load(object sender, EventArgs e)
        {
            double dias = 0;
            dias = Convert.ToDouble(GUI.CApp.GetParam("DVCR"));

            DateTime dt = new DateTime();
            dt = DateTime.Today;
            dt = dt.AddDays(dias);

            rowVencimiento.Properties.Value = dt;

            if (Program.Jornada == null)
            {
                GUI.CApp.ShowMessage("No puede crear certificados de regalo porque no tiene una jornada abierta", "Certificados de Regalo", true);
                btnAceptar.Enabled = false;
                Close();
            }
            
        }

        private void vGridControl1_ValidatingEditor(object sender, DevExpress.XtraEditors.Controls.BaseContainerValidateEditorEventArgs e)
        {
        }

        private void vGridControl1_FocusedRowChanged(object sender, DevExpress.XtraVerticalGrid.Events.FocusedRowChangedEventArgs e)
        {
            vGridControl1.ShowEditor();
        }
    }
}