﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace LLNet.Tienda.MainForms.Controls
{
    public partial class ctlFlexi : UserControl
    {
        public event System.EventHandler PayWithFlexi;
        public event System.ComponentModel.CancelEventHandler BeforePaywithFlexi;

        public decimal? OriginalValue;
        public decimal Rate = 1;
        public decimal? Currency;
        
        public ctlFlexi()
        {
            InitializeComponent();
            GUI.CApp.TraducirForm(this);
        }


        #region Eventos del Control

        private void ctlFlexi_Load(object sender, EventArgs e)
        {
            vGridControl.FocusedRow = rowMONTO;
        }


        private void vGridControl_FocusedRecordCellChanged(object sender, DevExpress.XtraVerticalGrid.Events.IndexChangedEventArgs e)
        {
            vGridControl.ShowEditor();
        }

        #endregion

        #region Procedimientos y funciones del controles
        
        public void AgregarPago(object sender, decimal pCurrencyValue,string pAutorizacion)
        {
            // Ejecuta el evento antes del pago en efectivo si se ha creado
            CancelEventArgs cea = new CancelEventArgs();
            if (BeforePaywithFlexi != null)
                BeforePaywithFlexi(sender, cea);

            // Detiene el proceso de pago con Flexi si se ha cancelado desde el evento
            if (cea.Cancel == true)
                return;

            if (pCurrencyValue == 0)
            {
                GUI.CApp.ShowMessage(GUI.CApp.TraducirPalabra("El monto a abonar debe ser mayor a cero"), catFlexi.Properties.Caption.ToString(), false);
                vGridControl.FocusedRow = rowVoucher;
                return;
            }

            // Obtiene los valores de pago
            decimal pValue = (OriginalValue < pCurrencyValue * Rate) ? (decimal)OriginalValue : pCurrencyValue * Rate;
            decimal pPay = pCurrencyValue * Rate;

            //moneda colones 188
            string mond = "188";
            //moneda dolares 840
            if (Currency == 2)
                mond = "840";
            //
            string Mto = FmtMonto(pCurrencyValue);
            decimal mont = decimal.Round(pCurrencyValue, 2);

            string documento = Program.FrmMainPos.DsPedido.WMS_DOCUMENTOS[0].DOCUMENTO.ToString();

            // Paga con Flexi
            Flexi(pValue, pPay, Rate, Currency, pCurrencyValue, pAutorizacion);

            // Ejecuta el evento despues del pago con credix
            if (PayWithFlexi != null)
                PayWithFlexi(sender, null);
        }

        public void Flexi(decimal pValue, decimal pPay,decimal pRate, decimal? pCurrency, decimal pCurrencyValue,string pAutorizacion)
        {
            Forms.frmPayments payment = ((Forms.frmPayments)FindForm());

            if (Convert.ToDecimal(payment.rowTotal.Properties.Value) < Convert.ToDecimal(payment.rowAbonado.Properties.Value) + pPay)
            {
                if (GUI.CApp.ShowMessage("El monto ingresado genera vuelto, ¿está seguro de ingresar este pago?", "Cobros y Facturación", true, true) == DialogResult.Cancel)
                    return;
            }

            if  (Convert.ToString(rowVoucher.Properties.Value) ==  string.Empty )
            {
                GUI.CApp.ShowMessage(GUI.CApp.TraducirPalabra("Debe ingresar el numero de voucher"), catFlexi.Properties.Caption.ToString(), false);
                return;
            }

            if (payment.beforePay(pValue) == -1) return;

            Model.PDV.DsPDV.WMS_PAGO_DOCUMENTOSRow pdr = payment.DsRecibo.WMS_PAGO_DOCUMENTOS.NewWMS_PAGO_DOCUMENTOSRow();

            pdr.DOCUMENTO = payment.DsRecibo.WMS_DOCUMENTOS[0].DOCUMENTO;
            pdr.TIPO_PAGO = 35;
            Utils.QueryParams qp = new LLNet.Utils.QueryParams();
            qp.AddReplaceParam("DOCUMENTO", (payment.DsRecibo.WMS_DOCUMENTOS[0].DOCUMENTO).ToString());
            pdr.LINEA = Convert.ToDecimal(GUI.CApp.bll.GetSingleValue("SiguienteLineaPago", qp, "PDV")) + payment.DsRecibo.WMS_PAGO_DOCUMENTOS.Rows.Count;
            pdr.VALOR = pValue;
            pdr.PAGADO = pPay;
            pdr.FECHA = DateTime.Now;
            pdr.AUTORIZACION = pAutorizacion;
            pdr.TASA_CAMBIO = pRate;



            if (pCurrency != null)
                pdr.MONEDA = (decimal)pCurrency;

            pdr.VALOR_MONEDA = pCurrencyValue;
            payment.DsRecibo.WMS_PAGO_DOCUMENTOS.AddWMS_PAGO_DOCUMENTOSRow(pdr);
        }


        private string FmtMonto(decimal pmonto)
        {
            decimal mont = decimal.Round(pmonto, 2); //redondea a 2 decimales
            return FmtMonto(mont.ToString());
        }

        private string FmtMonto(string pmonto)
        {
            string monto = string.Empty;
            Int32 punto = pmonto.LastIndexOf('.');
            Int32 lngtd = pmonto.Length;
            Int32 cdec = lngtd - punto;
            if (punto == -1)
            {//no trae punto    
                monto = pmonto.ToString() + "00";
            }
            else
            {
                if (lngtd == 1)
                {//punto al final
                    monto = pmonto.ToString().Remove(punto, 1) + "00";
                }
                else
                {
                    if (lngtd == 2)
                    {//punto x.x    
                        monto = pmonto.ToString().Remove(punto, 1) + "0";
                    }
                    else
                    {
                        if (lngtd == 3)
                        {//punto x.x    
                            monto = pmonto.ToString().Remove(punto, 1);
                        }
                    }
                }
            }
            return monto;
        }

        
        #endregion



    }
}


