﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LLNet.Utils;
using LLNet.GUI;

namespace LLNet.Tienda.MainForms.Forms.CierreConsol
{
    public partial class FrmModDepositos : Form
    {
        private string vUsuario = String.Empty;

        public FrmModDepositos()
        {
            InitializeComponent();
        }

        #region Propiedades

        public string Usuario
        {
            set
            {
                this.vUsuario = value;
            }
        }

        #endregion

        #region mLimpiar
        /// <summary>
        /// Limpia los campos de texto
        /// </summary>
        private void mLimpiar()
        {
            txtMoneda.Text = String.Empty;
            txtNoDepositoN.Text = String.Empty;
            txtNoPlanillaN.Text = String.Empty;
        }
        #endregion

        #region mModNoDepositos
        /// <summary>
        /// Modifica un número de depósito, de una seria de depósitos
        /// </summary>
        private void mModNoDepositos()
        {
            Utils.QueryParams vParametros = new QueryParams();
            vParametros.AddReplaceParam("NO_DEPOSITO", txtNoDepositoN.Text);
            vParametros.AddReplaceParam("NO_DEPOSITOV", cbxNoDepositos.Text);
            vParametros.AddReplaceParam("JORNADA", Program.Jornada.ToString());
            vParametros.AddReplaceParam("USUARIO", vUsuario);
            vParametros.AddReplaceParam("MONEDA", txtMoneda.Text);

            GUI.CApp.bll.ExecuteSql("WMS_P_MODIF_NUMDEPOSITO", vParametros, "PDV");            
        }
        #endregion

        #region mModNoPlanillas
        /// <summary>
        /// Modifica un número de planilla, de una seria de depósitos
        /// </summary>
        private void mModNoPlanillas()
        {
            Utils.QueryParams vParametros = new QueryParams();
            vParametros.AddReplaceParam("NO_PLANILLA", txtNoPlanillaN.Text);
            vParametros.AddReplaceParam("NO_PLANILLAV", cbxNoDepositos.SelectedValue.ToString());
            vParametros.AddReplaceParam("JORNADA", Program.Jornada.ToString());
            vParametros.AddReplaceParam("USUARIO", vUsuario);
            vParametros.AddReplaceParam("MONEDA", txtMoneda.Text);

            GUI.CApp.bll.ExecuteSql("WMS_P_MODIF_NUMPLANILLA", vParametros, "PDV");
        }
        #endregion

        #region mMostrarDatos
        /// <summary>
        /// Muestra los datos del depósito seleccionado, en los textbox del formulario
        /// </summary>
        private void mMostrarDatos()
        {
            mLimpiar();

            txtNoDepositoN.Text = cbxNoDepositos.Text;
            txtNoPlanillaN.Text = cbxNoDepositos.SelectedValue.ToString();

            DataRow vFila = ((DataTable)cbxNoDepositos.DataSource).Rows[cbxNoDepositos.SelectedIndex];

            // Obtiene la moneda del depósito y la planilla
            txtMoneda.Text = vFila[2].ToString();
        }
        #endregion

        #region mObtenerDepositos
        /// <summary>
        /// Obtiene la moneda, el número de depósito y planilla
        /// </summary>
        private void mObtenerDepositos()
        {
            cbxNoDepositos.DataSource = null;
            Utils.QueryParams vParametros = new QueryParams();
            vParametros.AddReplaceParam("JORNADA", Program.Jornada.ToString());

            cbxNoDepositos.SelectedIndexChanged -= new EventHandler(cbxNoDepositos_SelectedIndexChanged);

            // Obtiene los datos
            using (DataSet vdatos = GUI.CApp.bll.getDataSet("PDV", "DEPOSITOS", vParametros))
            {
                // Verifica que tenga datos
                if (vdatos.Tables.Count > 0 && vdatos.Tables[0].Rows.Count > 0)
                {
                    // Agrega los datos al grid activo
                    cbxNoDepositos.DataSource = vdatos.Tables[0];
                    cbxNoDepositos.DisplayMember = "NO_DEPOSITO";
                    cbxNoDepositos.ValueMember = "NO_PLANILLA";
                }
            }

            cbxNoDepositos.SelectedIndexChanged += new EventHandler(cbxNoDepositos_SelectedIndexChanged);
            cbxNoDepositos_SelectedIndexChanged(null, null);
        }
        #endregion

        #region btnSalir_Click
        private void btnSalir_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }
        #endregion

        #region FrmModDepositos_Load
        private void FrmModDepositos_Load(object sender, EventArgs e)
        {
            try
            {
                mObtenerDepositos();
            }
            catch (Exception ex)
            {
                CApp.ShowMessage(ex.Message, "Modificación Depósito", true);
            }
        }
        #endregion

        #region btnAceptar_Click
        private void btnAceptar_Click(object sender, EventArgs e)
        {
            bool vModificar = false;
            try
            {
                if (txtNoDepositoN.Text == String.Empty)
                {
                    CApp.ShowMessage("Ingrese el nuevo número de depósito", "Modificación Depósito", true);
                    txtNoDepositoN.Focus();
                }
                else if (txtNoPlanillaN.Text == String.Empty)
                {
                    CApp.ShowMessage("Ingrese el nuevo número de planilla", "Modificación Depósito", true);
                    txtNoPlanillaN.Focus();
                }
                else
                {
                    // Verifica si se debe actualizar el número de depósito
                    if (txtNoDepositoN.Text != cbxNoDepositos.Text)
                    {
                        vModificar = true;
                        mModNoDepositos();
                    }

                    // Verifica si se debe actualizar el número de planilla
                    if (txtNoPlanillaN.Text != cbxNoDepositos.SelectedValue.ToString())
                    {
                        vModificar = true;
                        mModNoPlanillas();
                    }

                    if (vModificar)
                    {
                        CApp.ShowMessage("Modificación registrada con éxito", "Modificación Depósito", true);
                        this.Dispose();
                    }
                    else
                    {
                        this.Dispose();
                    }
                }
            }
            catch (Exception ex)
            {
                CApp.ShowMessage(ex.Message, "Modificación Depósito", true);
            }
        }
        #endregion

        #region FrmModDepositos_KeyDown
        private void FrmModDepositos_KeyDown(object sender, KeyEventArgs e)
        {
            try
            {
                switch (e.KeyCode)
                {
                    case Keys.F12:
                        btnAceptar_Click(null, null);
                        break;
                    case Keys.Escape:
                        btnSalir_Click(null, null);
                        break;
                }
            }
            catch (Exception ex)
            {
                CApp.ShowMessage(ex.Message, "Modificación Depósito", true);
            }
        }
        #endregion

        #region cbxNoDepositos_SelectedIndexChanged
        private void cbxNoDepositos_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                mMostrarDatos();
            }
            catch (Exception ex)
            {
                CApp.ShowMessage(ex.Message, "Modificación Depósito", true);
            }
        }
        #endregion
    }
}